package PageImExporter::Theme;
use strict;

use MT;
use MT::Page;
use Data::Dumper;
sub condition {
    my ( $blog ) = @_;

    return 0 if MT->version_number < 5.1;

    my $page = MT->model('page')->load({ blog_id => $blog->id }, { limit => 1 });
    return defined $page ? 1 : 0;
}

sub template {
    my $app = shift;
    my ( $blog, $saved ) = @_;

    my @pages = MT->model('page')->load({
        blog_id => $blog->id,
    });
    return unless scalar @pages;
    my @list;
    my %checked_ids;
    if ( $saved ) {
        %checked_ids = map { $_ => 1 } @{ $saved->{plugin_default_pages_export_ids} };
    }
    for my $page ( @pages ) {
        push @list, {
            page_title  => $page->title,
            page_id     => $page->id,
            checked      => $saved ? $checked_ids{ $page->id } : 1,
        };
    }
    my %param = ( pages => \@list );

    my $template_file;
    if (MT->version_number < 5.1) {
        $template_file = 'export_page_50.tmpl';
    } else {
        $template_file = 'export_page.tmpl';
    }
    my $plugin = MT->component('PageImExporter');
    return $plugin->load_tmpl($template_file, \%param);
}

sub export {
    my ( $app, $blog, $settings ) = @_;
    my @pages;
    if ( defined $settings ) {
        my @ids = $settings->{plugin_default_pages_export_ids};
        @pages = MT->model('page')->load({ id => \@ids });
    }
    else {
        @pages = MT->model('page')->load({ blog_id => $blog->id });
    }
    return unless scalar @pages;

    # for customfield
    my @fields;
    my $iter = CustomFields::Field->load_iter(
           { blog_id => $blog->id,
             obj_type => 'page',
           });
    while (my $field = $iter->()) {
        push(@fields, $field->basename);
    }

    my $data = {};
    for my $page ( @pages ) {
        my $folder = $page->folder;
        my $path = $folder->basename if $folder;
        do {
            $folder = $folder && $folder->parent ?
                MT->model('folder')->load($folder->parent) : undef;
            $path = join "/", $folder->basename, $path if $folder;
        } while ($folder);

        my @obj_assets = MT::ObjectAsset->load(
            { object_ds => 'entry', object_id => $page->id } );
        my @obj_asset_ids = map { $_->asset_id } @obj_assets;
        my @obj_asset_labels;
        for my $asset_id (@obj_asset_ids) {
            my $asset = MT::Asset->load( $asset_id );
            push @obj_asset_labels, $asset->label if $asset;
        }

        # for customfield
        my %fields;
        for my $field (@fields) {
            my $data2 = $page->meta('field.'.$field);
            my $cf = CustomFields::Field->load( { blog_id  => $blog->id,
                                                  basename => $field,
                                                  obj_type => 'page' } );
            if ($cf->type eq 'image' ||
                $cf->type eq 'file'  ||
                $cf->type eq 'audio' ||
                $cf->type eq 'video') {
                if ($data2 =~ /.*asset-id="(\d+)".*/) {
                    my $asset_id = $1;
                    my $asset = MT::Asset->load( $asset_id );
                    if ($asset) {
                        $fields{'field.'.$field} = $asset->label;
                    } else {
                        $fields{'field.'.$field} = '';
                    }
                 }
            } else {
                $fields{'field.'.$field} = $data2;
            }
        }
        my $hash = {
            title => $page->title,
            text => $page->text,
            text_more => $page->text_more,
            excerpt => $page->excerpt,
            keywords => $page->keywords,
            convert_breaks => $page->convert_breaks,
            status => $page->status,
            authored_on => $page->authored_on,
            created_on => $page->created_on,
            modified_on => $page->modified_on,
            allow_comments => $page->allow_comments,
            allow_pings => $page->allow_pings,
            basename => $page->basename,
            tags  => join(',', $page->get_tags),
            folder  => $path,
            asset_labels  => join(',', @obj_asset_labels) || '',
            %fields,
        };
        $data->{ $page->id } = $hash;
    }
    return %$data ? $data : undef;
}

sub import {
    my ( $element, $theme, $obj_to_apply ) = @_;
    my $entries = $element->{data};
    _add_entries( $theme, $obj_to_apply, $entries, 'page' )
        or die "Failed to create theme default Pages";
    return 1;
}

sub _add_entries {
    my ( $theme, $blog, $pages, $class ) = @_;
    my $app = MT->instance;

    my $plugin = MT->component('PageImExporter');
    my $display = $plugin->translate('display');

    my @text_fields = qw(
        title   text     text_more
        excerpt keywords
    );
    PAGE: for my $id ( keys %$pages ) {
        my $page = $pages->{$id};
        my $iter = MT->model($class)->load_iter({
            basename => $page->{basename},
            blog_id  => $blog->id,
        });

        # check same basename
        while (my $p = $iter->()) {
            my $folder = $p->folder;
            my $path = $folder->basename if $folder;
            do {
                $folder = $folder && $folder->parent ?
                    MT->model('folder')->load($folder->parent) : undef;
                $path = join "/", $folder->basename, $path if $folder;
            } while ($folder);
            next PAGE if $path eq $page->{folder};
        }

        next if MT->model($class)->count({
            title => $page->{title},
            blog_id  => $blog->id,
        });
        my $obj = MT->model($class)->new();
        my $current_lang = MT->current_language;
        MT->set_language($blog->language);
        $obj->set_values({
            map { $_ => $theme->translate_templatized( $page->{$_} ) }
            grep { exists $page->{$_} }
            @text_fields
        });
        MT->set_language( $current_lang );

        $obj->basename( $page->{basename} );
        $obj->blog_id( $blog->id );
        $obj->author_id( $app->user->id );

        $obj->convert_breaks( $page->{convert_breaks} );
        $obj->authored_on( $page->{authored_on} );
        $obj->created_on( $page->{created_on} );
        $obj->modified_on( $page->{modified_on} );
        $obj->allow_comments( $page->{allow_comments} );
        $obj->allow_pings( $page->{allow_pings} );
        $obj->status(
            exists $page->{status} ? $page->{status} : MT::Entry::RELEASE()
        );
        if ( my $tags = $page->{tags} ) {
            my @tags = ref $tags ? @$tags : split( /\s*\,\s*/, $tags );
            $obj->set_tags( @tags );
        }

        # for customfield
        for my $name ( keys %$page ) {
            my $basename;
            if ( $name =~ /^field\.(.*)/ ) {
                $basename = $1;
                my $cf = CustomFields::Field->load({ blog_id => $blog->id,
                                                     basename => $basename,
                                                     obj_type => 'page'
                                                   });
                if ($cf) {
                    my $data = $page->{'field.' . $basename};
                    if ($cf->type eq 'image' ||
                        $cf->type eq 'file'  ||
                        $cf->type eq 'audio' ||
                        $cf->type eq 'video') {
                        my $asset = MT::Asset->load( { class => '*', blog_id => $blog->id, label => $data } );
                        next unless $asset;
                        my $asset_id = $asset->id;
                        my $asset_url = $asset->url;
                        my $asset_filename = $asset->file_name;
                        if ($cf->type eq 'image') {
                            $data = qq{<form mt:asset-id="$asset_id" class="mt-enclosure mt-enclosure-image" style="display: inline;"><a href="$asset_url">$display</a></form>};
                        } elsif ($cf->type eq 'file') {
                            $data = qq{<form mt:asset-id="$asset_id" class="mt-enclosure mt-enclosure-file" style="display: inline;"><a href="$asset_url">$asset_filename</a></form>};
                        } elsif ($cf->type eq 'audio') {
                            $data = qq{<form mt:asset-id="$asset_id" class="mt-enclosure mt-enclosure-audio" style="display: inline;"><a href="$asset_url">$asset_filename</a></form>};
                        } elsif ($cf->type eq 'video') {
                            $data = qq{<form mt:asset-id="$asset_id" class="mt-enclosure mt-enclosure-video" style="display: inline;"><a href="$asset_url">$asset_filename</a></form>};
                        }
                    }
                    $obj->meta('field.' . $basename, $data);
                }
            }
        }
        $obj->save or die $obj->errstr;

        my $path_str;
        if ( $class eq 'page' && ($path_str = $page->{folder}) ) {
            my @paths = split( '/', $path_str );
            my ($current, $parent);
            PATH: while ( my $path = shift @paths ) {
                my $terms = {
                    blog_id  => $blog->id,
                    basename => $path,
                };
                $terms->{parent} = $parent->id if $parent;
                $current = MT->model('folder')->load($terms);
                if ( !$current ) {
                    unshift @paths, $path;
                    while ( my $new = shift @paths ) {
                        my $f = MT->model('folder')->new();
                        $f->set_values({
                            blog_id   => $blog->id,
                            author_id => $app->user->id,
                            label     => $new,
                            basename  => $new,
                        });
                        $f->parent( $parent->id ) if $parent;
                        $f->save;
                        $parent = $f;
                    }
                    last PATH;
                }
                $parent = $current;
            }
            my $place = MT->model('placement')->new;
            $place->set_values({
                blog_id     => $blog->id,
                entry_id    => $obj->id,
                category_id => $parent->id,
                is_primary  => 1,
            });
            $place->save;
        }
        my @asset_labels = split(',', $page->{asset_labels});
        _add_asset_to_page( $blog->id, $obj->id, \@asset_labels ) if $page->{asset_labels};
    }
    1;
}

sub _add_asset_to_page {
    my ($blog_id, $id, $assets) = @_; # no nessesary class_type

    my $obj_assets = ();
    my @obj_assets = MT::ObjectAsset->load({ object_ds => 'entry', object_id => $id });
    foreach my $obj_asset (@obj_assets) {
        my $asset_id = $obj_asset->asset_id;
        $obj_assets->{$asset_id} = 1;
    }
    my $seen = ();
    unless ( ref $assets ) {
        push @{$assets}, $assets;
    }

    foreach my $asset_label (@{$assets}) {
        my $asset = MT::Asset->load( { class => '*', blog_id => $blog_id, label => $asset_label } );
        next unless $asset;
        my $obj_asset = MT::ObjectAsset->load({ asset_id => $asset->id, object_ds => 'entry', object_id => $id });
        unless ($obj_asset) {
            my $obj_asset = new MT::ObjectAsset;
            $obj_asset->blog_id($blog_id);
            $obj_asset->asset_id($asset->id);
            $obj_asset->object_ds('entry');
            $obj_asset->object_id($id);
            $obj_asset->save;
        }
        $seen->{$asset->id} = 1;
    }
    foreach my $asset_id (keys %{$obj_assets}) {
        unless ($seen->{$asset_id}) {
            my $obj_asset = MT::ObjectAsset->load({ asset_id => $asset_id, object_ds => 'entry', object_id => $id });
            $obj_asset->remove;
        }
    }
    return 1;
}

sub info {
    my ( $element, $theme, $blog ) = @_;
    my $data = $element->{data};

    return sub {
        MT->translate( 'Pages' ) .'('. MT->translate( '[_1] pages', scalar keys %{$element->{data}} ) .')';
    };
}

1;
